///////////////////////////////////////////////////////////////////////////
// header file: EventContainer
// 
// Description: 
//  read in variables from the EventTree chains and make them available
//  in Object form.
//
// Author: Reinhard Schwienhorst
// Date: 10/19/2006
//
// Modifications:
// 10/19/2006, RS: - initial version only with electrons
//
//
////////////////////////////////////////////////////////////////////////////
#ifndef particles_h
#define particles_h

#include <vector>
#include <string>
#include <iostream>

// MakeClass output:
#include "single_top_analysis/CollectionTree.hpp"
#include "single_top_analysis/EventTree.hpp"
#include "single_top_analysis/TruthTree.hpp"
// our own particles
#include "single_top_analysis/Electron.hpp"
#include "single_top_analysis/Muon.hpp"
#include "single_top_analysis/Tau.hpp"
#include "single_top_analysis/Jet.hpp"
// some MC particles
#include "single_top_analysis/MCParticle.hpp"
//
#include <TObjArray.h>
#include <TLorentzVector.h>
#include "TEnv.h"
//


class EventContainer
{
 public:
  //default constructor
  EventContainer();
  //destructor
  ~EventContainer();

  // Set the name of the source that we are running over
  void SetSourceName(const TString& name) { _sourceName=name; };
  TString GetSourceName() const { return _sourceName; };

  // Set the number for the source that we are running over
  void SetSourceNumber(Int_t nu) { _sourceNumber=nu; };
  Int_t GetSourceNumber() const { return _sourceNumber; };

  // Set the target top mass for top quark constraint in best jet algorithm
  void SetTargetTopMass(const Double_t& mass) { _targetTopMass=mass; };
  Double_t GetTargetTopMass() const { return _targetTopMass; };

  // initialize this class 
  void initialize(CollectionTree* cotr,EventTree* evtr, TruthTree* trtr);

  // Create TLorentz Vectors for the particles in the interaction
  int readEvent();

  // Create final state particles and make cuts if necessary (require at least
  // two jets and at least one lepton)
  // return false if these conditions are not fulfilled
  bool CreateFinalState();

  // reconstruct the top in several different ways from the final state objects
  void makeTopQuarks();

  // Set up the particles for the next event
  int GetNextEvent();
  // initialize the event counter to Get a very specific event in the chain
  inline void SetEventCount(int i) { _eventCount=i; };


  // Set and Read the flag that determines whether 
  // Also determine whether we should only process leptons
  // of a specific flavor (0 to process all).
  inline void SetLeptonSelect(int flavor) { _leptonSelect=flavor; };
  inline int GetLeptonSelect() { return _leptonSelect; };

  // Set a normalization factor for all histograms
  inline void SetNormalization(double norm) { _normalization=norm; };

  // Get the actual tree
  inline CollectionTree* GetCollectionTree() const { return _collectionTree; };
  // Get the actual tree
  inline EventTree* GetEventTree() const { return _eventTree; };
  // Get the actual tree
  inline TruthTree* GetTruthTree() const { return _truthTree; };
  
  //
  // debugging
  inline void SetDebugLevel(int i) { _debugLevel=i; };
  inline int GetDebugLevel() const { return _debugLevel; };

  //
  // handle event weights
  // Set the global event weight
  void SetGlobalEventWeight(const Double_t& wt) { _globalEventWeight=wt; };
  // set the output event weight.
  void SetOutputEventWeight(const Double_t& wt) { _outputEventWeight=wt; };

  // get the global event weight and others for filling histograms
  Double_t GetEventWeight() const { return _outputEventWeight; };
  Double_t GetOutputEventWeight() const { return _outputEventWeight; };
  Double_t GetTreeEventWeight() const { return _treeEventWeight; };
  Double_t GetGlobalEventWeight() const { return _globalEventWeight; };

  // get the configuration 
  TEnv * GetConfig() {return &_config; };

  ////////////////////////////////////
  // object helpers:
  //

  ////////////////////////////////////
  // make the particles available for every event.
  // partons:
  int runNumber;              // run number from the root tree
  int eventNumber;            // event number from the root tree

  // reconstructed particles
  std::vector<Electron> electrons;
  std::vector<Muon>     muons;
  std::vector<Tau>      taus;
  std::vector<Jet>      jets;
  // the full list of all MC particles
  std::vector<MCParticle> MCparticles;
  // ..... add more here

  /////////////////////////////////
  // protected data members are available to classes that inherit from me
protected:
  
  //
  // debugging level
  int _debugLevel;
  // use TEnv to read configuration parameters
  TEnv _config;


  
private:
  
  //////////////////////////////
  // reconstruction information
  //
  //
  // the target top mass used in the best jet algorithm
  Double_t _targetTopMass;
  // 

  //////////////////////////////////////////
  // The classes that contain the information from the trees:
  CollectionTree *_collectionTree;
  EventTree *_eventTree;
  TruthTree *_truthTree;

  //
  // should we only process certain events?
  int _leptonSelect;     // = 11: electrons, =12: only muons, =0: all

  //
  // Normalization factor for histograms
  Double_t _normalization; // additional normalization factor

  //
  // counters to keep track of where we are in the chain
  int _eventCount;

  //
  // name and number of the source that we are reading (i.e. tb, tq, tW, tt_lepjets, tt_dilep, etc.
  TString _sourceName;
  Int_t   _sourceNumber;

  // keep track of several event weights
  Double_t _globalEventWeight;         // the global event weight, unchanging event to event
  Double_t _treeEventWeight;           // the event weight read in from the tree
  Double_t _outputEventWeight;         // the output event weight passed to the histograms



  //
  ////////////////////////////////////////
  // needed to run with Root:
  ClassDef(EventContainer,0)
};


#endif
