/******************************************************************************
 * CutListProcessor.cpp                                                       *
 *                                                                            *
 * Process the list of cuts and histogramming classes                         *
 *                                                                            *
 * Derived from BaseCut                                                       *
 * Parent to AnalysisMain                                                     *
 *                                                                            *
 * Public Member Functions of CutNJet class                                   *
 *    CutListProcessor()                -- Parameterized Constructor          *
 *    ~CutListProcessor()               -- Destructor                         *
 *    AddCut()                          -- Add a cut to the list to process   *
 *    BookHistogram()                   -- Book Histograms                    *
 *    Apply()                           -- Make cuts and fill histograms      *
 *    GetCutName()                      -- Get name of cut                    *
 *    PrintCut()                        -- Print name of cut                  *
 *    SetDebug()                        -- Set Debug on or off                *
 *    DeclareTH2F()                     -- Wrapper for TH2F                   *
 *    AddDefaultBaseCut                 -- Add template to every CutListProc  *
 *    template<class T>                 -- Template to be added to every list *
 *    GetCurrentPackage()               -- Returns current package            *
 *    GetCurrentEventContainer()        -- Returns event read                 *
 *                                                                            *
 * Private Data Members of CutNJet class                                      *
 *    string name                         -- Name of CutListProcessor         *
 *    vector<BaseCut *> _vectorCutAndHist -- Vector of cuts and histograms    *
 *    TH1F *_hCutAndHist                  -- Number of times each cut runs    *
 *    Bool_t _alwaysPass                  -- Always returns kFALSE            *
 *    CleanUpRun()                        -- Clean up after loop              *
 *    Bool_t _gDegub                      -- If debug is on                   *
 *                                                                            *
 *    template <class T> class AutoInsert_t                                   *
 *    template <class T> static AutoInsert_t<T> *AutoInsert_t_maker           *
 *    static std::vector<AutoInsert*> &defaultCuts(void)                      *
 *    static BaseCut *_currentBaseCut;                                        *
 *    static EventContainer *_currentEventContainer;                          *
 *                                                                            *
 * History                                                                    *
 *      10 Oct 2006 - RS: Ported D0/theory original to Atlas.                 *
 *      14 Nov 2006 - Modified by P. Ryan - cleanup and reorganized           *
 *****************************************************************************/

#include "single_top_analysis/CutListProcessor.hpp"

#include "single_top_analysis/myTH1F.hpp"
#include "TDirectory.h"

#include <sstream>
#include <iostream>
#include <iomanip>
#include <string>
#include <algorithm>
#include <functional>

// Set Debug Flag
Bool_t CutListProcessor::_gDebug = kFALSE;

// Set _currentEventContainer and _currentBaseCut to NULL
EventContainer *CutListProcessor::_currentEventContainer = NULL;
BaseCut *CutListProcessor::_currentBaseCut = NULL;

// Set default cuts (applied to all CutListProcessors)
std::vector<CutListProcessor::AutoInsert*> &CutListProcessor::defaultCuts()
{
  static std::vector<CutListProcessor::AutoInsert*> gCuts;
  return gCuts;
} //defaultCuts


/**********************************************************************************
 * CutListProcessor::CutListProcessor (const std::string &name, Bool_t alwaysPass) *
 *                                                                                 *
 * Parameterized Constructor                                                       *
 *                                                                                 *
 * Input:  Name of class and a flag                                                *
 * Output: None                                                                    *
 ***********************************************************************************/
CutListProcessor::CutListProcessor (const std::string &name, Bool_t alwaysPass)
  : _name (name), _alwaysPass (alwaysPass)
{
  std::for_each (defaultCuts().begin(), defaultCuts().end(),
		 std::bind2nd (std::mem_fun (&AutoInsert::AddCuts), this));
} //CutListProcessor()

/******************************************************************************
 * CutListProcessor::GetCutName()                                             *
 *                                                                            *
 * Return name of CutListProcessor                                            *
 *                                                                            *
 * Input:  None                                                               *
 * Output: Returns name                                                       *
 ******************************************************************************/
std::string CutListProcessor::GetCutName()
{
  return _name;
} //GetCutName

//
// BookHistogram
//
//  Pass the booking on down! We don't have any of them ourselves for now!
//

/******************************************************************************
 * void CutListBrocessor::BookHistogram()                                     *
 *                                                                            *
 * Book Histograms                                                            *
 * Book the histogram giving the number of times the cuts and filling ran     *
 * Also book histograms                                                       *
 * The booking of the other histograms is done in the derived classes         *
 *                                                                            *
 * Input:  None                                                               *
 * Output: Prints each cuts/histogram class processes                         *
 ******************************************************************************/
void CutListProcessor::BookHistogram ()
{
  //
  // Let em know what happened. :-)
  //
  //  Watch for the case that we have no sub-cuts running. In this case
  // don't generate the who ran histo.
  //
  
  // Directory name of cut/hist to be constructed from steram
  std::ostringstream directoryNameSteam;
  // Directory name of cut/hist as a string
  std::string directoryName;

  // Only book histograms if cuts/histogram classes exist in vector
  if (_vectorCutAndHist.size() > 0) {

    // Book histograms giving the number of time the cuts and filling classes run
    _hCutAndHist = new TH1F ("CutAndHist", "Number of times each cut and histogram class ran",
			     _vectorCutAndHist.size(), 0, _vectorCutAndHist.size());
    _hCutAndHist -> SetXTitle("Cut/Hist");
    _hCutAndHist -> SetYTitle("Events");
    
    //
    // Now, book all the guys below that....
    //
    
    // Initialize number of Cut/Hist classes
    Int_t numCutAndHist = 0;
    
    // Write to screen
    std::cout << "<CutListProcessor::BookHistogram>      --------------------------------  " << std::endl;
    std::cout << "<CutListProcessor::BookHistogram>     |   Cut & Histo Classes Called   | " << std::endl;
    std::cout << "<CutListProcessor::BookHistogram>      --------------------------------  " << std::endl;
    std::cout.setf(std::ios::left);

    // Loop over cuts/hists in Vector
    for(std::vector<BaseCut *>::iterator itask = _vectorCutAndHist.begin(); itask != _vectorCutAndHist.end(); ++itask){
      // Set axis label
      _hCutAndHist -> GetXaxis() -> SetBinLabel( numCutAndHist+1, (*itask) -> GetCutName().c_str() );

      // Save directory name as string
      directoryNameSteam.str("");
      directoryNameSteam << std::setw(2) << std::setfill('0') << numCutAndHist << std::setw(0) << "_" << (*itask)->GetCutName();
      directoryName = directoryNameSteam.str();
      
      // Write directory name to screen
      std::cout << "<CutListProcessor::BookHistogram>     |  " <<  std::setw(27) << directoryName << "   |" << std::endl;

      // Get directory of cut/hist and switch to that directory
      TDirectory *directory = GetDirectory() -> mkdir(directoryName.c_str()," ");
      directory -> cd();

      // Set directory of current cut/hist and book that histogram
      (*itask) -> SetDirectory(directory);
      (*itask) -> BookHistogram();

      // Increment counter
      numCutAndHist++;
    } //for
    std::cout.setf(std::ios::right);
    std::cout << "<CutListProcessor::BookHistogram>      -------------------------------- " << std::endl;

  } //if

} //BookHistogram()

/******************************************************************************
 * void CutListBrocessor::PrintCut()                                          *
 *                                                                            *
 * Print cut info                                                             *
 * Calls the derived classes for each cut/hist                                *
 *                                                                            *
 * Input:  String used to indent                                              *
 * Output: Prints info for cuts                                               *
 ******************************************************************************/
void CutListProcessor::PrintCut (const std::string &indent)
{
  // Call printing method at the end of the run. 
  // The PrintCut() method is usefully implemented in Selection class so far.
  //  std::cout << indent << "<CutListProcessor::PrintCut> Results from Cut List '" << _name << "':" << std::endl;
  for(std::vector<BaseCut *>::iterator itask=_vectorCutAndHist.begin(); itask!=_vectorCutAndHist.end(); ++itask){
    (*itask)->PrintCut(indent + "  ");
  } //for
} //PrintCut()


/******************************************************************************
 * void CutListBrocessor::Apply()                                             *
 *                                                                            *
 * Fills histogram for cut/hist information                                   *
 * Cleans up if necessary                                                     *
 *                                                                            *
 * Input:  EventContainer Object                                              *
 * Output: Flag kTRUE if successful, kFALSE if not                            *
 ******************************************************************************/
Bool_t CutListProcessor::Apply (EventContainer *EventContainerObj)
{
  // Copy current values to OLD
  BaseCut *oldBaseCut = _currentBaseCut;
  EventContainer *oldEventContainer = _currentEventContainer;

  try {
    
    // Set new to current
    _currentEventContainer = EventContainerObj;
    
    // Loop on all cuts/histograms added
    Int_t count = 0;
    for(std::vector<BaseCut *>::iterator itask = _vectorCutAndHist.begin(); itask != _vectorCutAndHist.end(); ++itask){
      // Debug info
      if (_gDebug) {
	std::cout << "<CutListProcessor::Apply> " << "DEBUG: Apply " << (*itask) -> GetCutName() << std::endl;
      } //if

      // Change directory and get current task
      (*itask) -> GetDirectory() -> cd();
      _currentBaseCut = *itask;
      
      // Fill histogram
      _hCutAndHist -> Fill(count);

      // Clean up and return kFALSE if no cut/hist exists in vector
      if ( !(*itask) -> Apply() ) {
	CleanUpRun (_vectorCutAndHist.begin(), itask+1);
	return _alwaysPass;
      } //if

      // Increment counter
      count++;
      
    }//for
    
    CleanUpRun (_vectorCutAndHist.begin(), _vectorCutAndHist.end());
    return kTRUE;
  } //try
  
  // Catch exception
  catch (...) {
    // Reset current to old
    _currentEventContainer = oldEventContainer;
    _currentBaseCut        = oldBaseCut;
    throw;
  } //catch

} //Apply()

/******************************************************************************
 * void CutListBrocessor::CleanUpRun()                                        *
 *                                                                            *
 * Calls DoneProcessing() for each element in Vector                          *
 *                                                                            *
 * Input:  Start and End of Vector                                            *
 * Output: None                                                               *
 ******************************************************************************/
void CutListProcessor::CleanUpRun(std::vector<BaseCut *>::iterator start,
				     std::vector<BaseCut *>::iterator end)
{
  while (start != end) {
    end--;
    (*end) -> DoneProcessingLoop();
  } //while
} //CleanUpRun

/******************************************************************************
 * void CutListBrocessor::AddCut()                                            *
 *                                                                            *
 * Add a cut/hist to Vector                                                   *
 *                                                                            *
 * Input:  Cut/hist class                                                     *
 * Output: None                                                               *
 ******************************************************************************/
void CutListProcessor::AddCut (BaseCut *CutAndHistBaseCut)
{
  _vectorCutAndHist.push_back(CutAndHistBaseCut);
} //AddCut()


