/******************************************************************************
 * HistogrammingJet.cpp                                                       *
 *                                                                            *
 * Books and fills histograms                                                 *
 * Used for events passing cuts applied in other classes                      *
 *                                                                            *
 * Derived from HistoCut which is in turn derived from BaseCut                *
 *                                                                            *
 * Public Member Functions of AnalysisMain class                              *
 *    HistogrammingJet()              -- Parameterized Constructor            *
 *    ~HistogrammingJet()             -- Destructor                           *
 *    BookHistogram()                   -- Book histograms                    *
 *    Apply()                           -- Fill histograms only (No Cuts)     *
 *    GetCutName()                      -- Returns "HistogrammingJet"         *
 *                                                                            *
 * Private Data Members of this class                                         *
 *  - lots of histograms of jet properties                                    *
 *                                                                            *
 * History                                                                    *
 *      14 Nov 2006 - Created by R. Schwienhorst for ATLAS                    *
 *****************************************************************************/

#include "single_top_analysis/HistogrammingJet.hpp"
#include<iostream>

using namespace std;

/******************************************************************************
 * HistogrammingJet::HistogrammingJet(EventContainer *obj)          *
 *                                                                            *
 * Parameterized Constructor                                                  *
 *                                                                            *
 * Input:  Event container                                                    *
 * Output: None                                                               *
 ******************************************************************************/
HistogrammingJet::HistogrammingJet(EventContainer *obj)
{
  SetEventContainer(obj);
} //HistogrammingJet()

/******************************************************************************
 * HistogrammingJet::~HistogrammingJet()                                  *
 *                                                                            *
 * Destructor                                                                 *
 *                                                                            *
 * Input:  None                                                               *
 * Output: None                                                               *
 ******************************************************************************/
HistogrammingJet::~HistogrammingJet()
{
  
} //HistogrammingJet

/******************************************************************************
 * void HistogrammingJet::BookHistogram()                                          *
 *                                                                            *
 * Book Histograms                                                            *
 *                                                                            *
 * Input:  None                                                               *
 * Output: None                                                               *
 ******************************************************************************/
void HistogrammingJet::BookHistogram(){
  
  // Histogram of Number of Objects
  _hNObj = DeclareTH1F("JetN","Number of Jets",10,0.,10.);
  _hNObj->SetXAxisTitle("N");
  _hNObj->SetYAxisTitle("Events");
  // Jet 1 Pt
  _hPtObj1 = DeclareTH1F("Jet1Pt","Jet 1 P_{T}",200,0.,400.);
  _hPtObj1->SetXAxisTitle("P_{T} (jet 1) [GeV]");
  _hPtObj1->SetYAxisTitle("Events");
  // Jet 1 Eta
  _hEtaObj1 = DeclareTH1F("Jet1Eta","Jet 1 Pseudorapidity",100,-5.,5.);
  _hEtaObj1->SetXAxisTitle("#eta (jet 1) ");
  _hEtaObj1->SetYAxisTitle("Events");
  // Jet 1 Phi
  _hPhiObj1 = DeclareTH1F("Jet1Phi","Jet 1 phi",100,-1.*TMath::TwoPi(),TMath::TwoPi());
  _hPhiObj1->SetXAxisTitle("#phi (jet 1)");
  _hPhiObj1->SetYAxisTitle("Events");
  // Jet 2 Pt
  _hPtObj2 = DeclareTH1F("Jet2Pt","Jet 2 P_{T}",200,0.,400.);
  _hPtObj2->SetXAxisTitle("P_{T} (jet 2) [GeV]");
  _hPtObj2->SetYAxisTitle("Events");
  // Jet 2 Eta
  _hEtaObj2 = DeclareTH1F("Jet2Eta","Jet 2 Pseudorapidity",100,-5.,5.);
  _hEtaObj2->SetXAxisTitle("#eta (jet 2) ");
  _hEtaObj2->SetYAxisTitle("Events");
  // Jet 2 Phi
  _hPhiObj2 = DeclareTH1F("Jet2Phi","Jet 2 phi",100,-1.*TMath::TwoPi(),TMath::TwoPi());
  _hPhiObj2->SetXAxisTitle("#phi (jet 2)");
  _hPhiObj2->SetYAxisTitle("Events");

  // any Jet Pt
  _hPt = DeclareTH1F("JetPt","any Jet P_{T}",200,0.,400.);
  _hPt->SetXAxisTitle("P_{T} (jet) [GeV]");
  _hPt->SetYAxisTitle("Events");
  // any Jet Eta
  _hEta = DeclareTH1F("JetEta","any Jet Pseudorapidity",100,-5.,5.);
  _hEta->SetXAxisTitle("#eta (jet) ");
  _hEta->SetYAxisTitle("Events");
  // any Jet Phi
  _hPhi = DeclareTH1F("JetPhi","Any jet phi",100,-1.*TMath::TwoPi(),TMath::TwoPi());
  _hPhi->SetXAxisTitle("#phi (jet)");
  _hPhi->SetYAxisTitle("Events");
  // any Jet Charge
  _hCharge = DeclareTH1F("JetCharge","Any jet charge",3,-1.5,1.5);
  _hCharge->SetXAxisTitle("charge (jet)");
  _hCharge->SetYAxisTitle("Events");
  // any Jet E over P
  _hEtEM0 = DeclareTH1F("JetEtEM0","Any jet E_{T} in EM layer 0",100,0.,100.);
  _hEtEM0->SetXAxisTitle("E_{T} (jet) [GeV]");
  _hEtEM0->SetYAxisTitle("Events");
  
  //cout<<"end of HistogrammingJet::BookHistogram"<<endl;

} //BookHistogram

/******************************************************************************
 * Bool_t HistogrammingJet::Apply()                                         *
 *                                                                            *
 * Fill histograms                                                            *
 *                                                                            *
 * Input:  None                                                               *
 * Output: kTrue if successful                                                *
 ******************************************************************************/
Bool_t HistogrammingJet::Apply()
{
  //cout<<"Begin of HistogrammingJet::Apply()"<<endl;
  
  // Get Event Tree
  EventContainer *evc = GetEventContainer();
  
  // Fill Histograms
  _hNObj -> Fill(evc->jets.size());

  if(evc->jets.size()>0) {
    _hPtObj1  -> Fill(evc->jets[0].Pt());
    _hEtaObj1 -> Fill(evc->jets[0].Eta());
    _hPhiObj1 -> Fill(evc->jets[0].Phi());
  }
  if(evc->jets.size()>1) {
    _hPtObj2  -> Fill(evc->jets[1].Pt());
    _hEtaObj2 -> Fill(evc->jets[1].Eta());
    _hPhiObj2 -> Fill(evc->jets[1].Phi());
  }

  // now loop over all jets and histogram their properties
  for(int io=0;io<evc->jets.size();io++) {
    _hPt  -> Fill(evc->jets[io].Pt());
    _hEta -> Fill(evc->jets[io].Eta());
    _hPhi -> Fill(evc->jets[io].Phi());
    _hCharge->Fill(evc->jets[io].GetCharge());
    _hEtEM0->Fill(evc->jets[io].GetEtEM0());
  }    

  //cout<<"End of HistogrammingJet::Apply()"<<endl;
  return kTRUE;  
  
} //Apply












