/******************************************************************************
 * HistogrammingMuon.cpp                                                      *
 *                                                                            *
 * Books and fills histograms                                                 *
 * Used for events passing cuts applied in other classes                      *
 *                                                                            *
 * Derived from HistoCut which is in turn derived from BaseCut                *
 *                                                                            *
 * Public Member Functions of AnalysisMain class                              *
 *    HistogrammingMuon()              -- Parameterized Constructor           *
 *    ~HistogrammingMuon()             -- Destructor                          *
 *    BookHistogram()                   -- Book histograms                    *
 *    Apply()                           -- Fill histograms only (No Cuts)     *
 *    GetCutName()                      -- Returns "HistogrammingMuon"        *
 *                                                                            *
 * Private Data Members of this class                                         *
 * - lots of histograms of muon properties                                    *
 *                                                                            *
 * History                                                                    *
 *      14 Nov 2006 - Created by R. Schwienhorst for ATLAS                    *
 *****************************************************************************/

#include "single_top_analysis/HistogrammingMuon.hpp"
#include<iostream>

using namespace std;

/******************************************************************************
 * HistogrammingMuon::HistogrammingMuon(EventContainer *obj)          *
 *                                                                            *
 * Parameterized Constructor                                                  *
 *                                                                            *
 * Input:  Event container                                                    *
 * Output: None                                                               *
 ******************************************************************************/
HistogrammingMuon::HistogrammingMuon(EventContainer *obj)
{
  SetEventContainer(obj);
} //HistogrammingMuon()

/******************************************************************************
 * HistogrammingMuon::~HistogrammingMuon()                                  *
 *                                                                            *
 * Destructor                                                                 *
 *                                                                            *
 * Input:  None                                                               *
 * Output: None                                                               *
 ******************************************************************************/
HistogrammingMuon::~HistogrammingMuon()
{
  
} //HistogrammingMuon

/******************************************************************************
 * void HistogrammingMuon::BookHistogram()                                          *
 *                                                                            *
 * Book Histograms                                                            *
 *                                                                            *
 * Input:  None                                                               *
 * Output: None                                                               *
 ******************************************************************************/
void HistogrammingMuon::BookHistogram(){
  
  // Histogram of Number of Objects
  _hNObj = DeclareTH1F("MuN","Number of Muons",10,0.,10.);
  _hNObj->SetXAxisTitle("N");
  _hNObj->SetYAxisTitle("Events");
  // Muon 1 Pt
  _hPtObj1 = DeclareTH1F("Mu1Pt","Muon 1 P_{T}",200,0.,400.);
  _hPtObj1->SetXAxisTitle("P_{T} (muon 1) [GeV]");
  _hPtObj1->SetYAxisTitle("Events");
  // Muon 1 Eta
  _hEtaObj1 = DeclareTH1F("Mu1Eta","Muon 1 Pseudorapidity",100,-5.,5.);
  _hEtaObj1->SetXAxisTitle("#eta (muon 1) ");
  _hEtaObj1->SetYAxisTitle("Events");
  // Muon 1 Phi
  _hPhiObj1 = DeclareTH1F("Mu1Phi","Muon 1 phi",100,-1.*TMath::TwoPi(),TMath::TwoPi());
  _hPhiObj1->SetXAxisTitle("#phi (muon 1)");
  _hPhiObj1->SetYAxisTitle("Events");
  // Muon 2 Pt
  _hPtObj2 = DeclareTH1F("Mu2Pt","Muon 2 P_{T}",200,0.,400.);
  _hPtObj2->SetXAxisTitle("P_{T} (muon 2) [GeV]");
  _hPtObj2->SetYAxisTitle("Events");
  // Muon 2 Eta
  _hEtaObj2 = DeclareTH1F("Mu2Eta","Muon 2 Pseudorapidity",100,-5.,5.);
  _hEtaObj2->SetXAxisTitle("#eta (muon 2) ");
  _hEtaObj2->SetYAxisTitle("Events");
  // Muon 2 Phi
  _hPhiObj2 = DeclareTH1F("Mu2Phi","Muon 2 phi",100,-1.*TMath::TwoPi(),TMath::TwoPi());
  _hPhiObj2->SetXAxisTitle("#phi (muon 2)");
  _hPhiObj2->SetYAxisTitle("Events");

  // any Muon Pt
  _hPt = DeclareTH1F("MuPt","any Muon P_{T}",200,0.,400.);
  _hPt->SetXAxisTitle("P_{T} (muon) [GeV]");
  _hPt->SetYAxisTitle("Events");
  // any Muon Eta
  _hEta = DeclareTH1F("MuEta","any Muon Pseudorapidity",100,-5.,5.);
  _hEta->SetXAxisTitle("#eta (muon) ");
  _hEta->SetYAxisTitle("Events");
  // any Muon Phi
  _hPhi = DeclareTH1F("MuPhi","Any muon phi",100,-1.*TMath::TwoPi(),TMath::TwoPi());
  _hPhi->SetXAxisTitle("#phi (muon)");
  _hPhi->SetYAxisTitle("Events");
  // any Muon Charge
  _hCharge = DeclareTH1F("MuCharge","Any muon charge",3,-1.5,1.5);
  _hCharge->SetXAxisTitle("charge (muon)");
  _hCharge->SetYAxisTitle("Events");
  // any Muon Etcone
  _hEtCone = DeclareTH1F("MuEtCone","Any muon cone E_{T}",100,0.,50.);
  _hEtCone->SetXAxisTitle("E_{T}^{cone} (muon)");
  _hEtCone->SetYAxisTitle("Events");
  
  //cout<<"end of HistogrammingMuon::BookHistogram"<<endl;

} //BookHistogram

/******************************************************************************
 * Bool_t HistogrammingMuon::Apply()                                         *
 *                                                                            *
 * Fill histograms                                                            *
 *                                                                            *
 * Input:  None                                                               *
 * Output: kTrue if successful                                                *
 ******************************************************************************/
Bool_t HistogrammingMuon::Apply()
{
  //cout<<"Begin of HistogrammingMuon::Apply()"<<endl;
  
  // Get Event Tree
  EventContainer *evc = GetEventContainer();
  
  // Fill Histograms
  _hNObj -> Fill(evc->muons.size());

  if(evc->muons.size()>0) {
    _hPtObj1  -> Fill(evc->muons[0].Pt());
    _hEtaObj1 -> Fill(evc->muons[0].Eta());
    _hPhiObj1 -> Fill(evc->muons[0].Phi());
  }
  if(evc->muons.size()>1) {
    _hPtObj2  -> Fill(evc->muons[1].Pt());
    _hEtaObj2 -> Fill(evc->muons[1].Eta());
    _hPhiObj2 -> Fill(evc->muons[1].Phi());
  }

  // now loop over all muons and histogram their properties
  for(int io=0;io<evc->muons.size();io++) {
    _hPt  -> Fill(evc->muons[io].Pt());
    _hEta -> Fill(evc->muons[io].Eta());
    _hPhi -> Fill(evc->muons[io].Phi());
    _hCharge->Fill(evc->muons[io].GetCharge());
    _hEtCone->Fill(evc->muons[io].GetEtCone());
  }    

  //cout<<"End of HistogrammingMuon::Apply()"<<endl;
  return kTRUE;  
  
} //Apply












